@@ -37,7 +37,7 @@ import {
   lock_requirements_with_pipenv
 } from '../thoth';
 
-// import { INotification } from 'jupyterlab_toastify';
+import { INotification } from 'jupyterlab_toastify';
 
 import {
   get_kernel_name
@@ -94,8 +94,10 @@ export interface IDependencyManagementUIState {
   error_msg: string,
   resolution_engine: string,
   thoth_timeout: number,
-  isEditing: boolean,
-  root_directory: string
+  thoth_force: boolean,
+  root_directory: string,
+  thoth_resolution_error_msg: string,
+  pipenv_resolution_error_msg: string
 }
 
 /**
@@ -142,8 +144,10 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         error_msg: undefined,
         resolution_engine: "thoth",
         thoth_timeout: 180,
-        isEditing: false,
-        root_directory: ""
+        thoth_force: false,
+        root_directory: "",
+        thoth_resolution_error_msg: undefined,
+        pipenv_resolution_error_msg: undefined
       }
 
       this.onStart = this.onStart.bind(this),
@@ -191,6 +195,8 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
       kernel_name: string,
       thoth_config: ThothConfig,
       error_msg?: string,
+      thoth_resolution_error_msg?: string,
+      pipenv_resolution_error_msg?: string,
     ) {
 
       var new_state: IDependencyManagementUIState = this.state
@@ -214,10 +220,20 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       _.set(new_state, "error_msg", error_msg)
 
+      _.set(new_state, "thoth_resolution_error_msg", thoth_resolution_error_msg)
+
+      _.set(new_state, "pipenv_resolution_error_msg", pipenv_resolution_error_msg)
+
       console.debug("new", new_state)
       this.setState(new_state);
     }
 
+    async setNewState(ui_state: IDependencyManagementUIState) {
+
+      console.log("new state", ui_state)
+      this.setState(ui_state);
+    }
+
     /**
      * Function: Set recommendation type for thamos advise
      */
@@ -229,9 +245,37 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
           recommendation_type: recommendation_type
         }
       );
-
     }
 
+    /**
+     * Function: Change recommendation type for thamos advise
+     */
+
+    changeRecommendationType(event: React .ChangeEvent<HTMLInputElement>) {
+
+      const recommendation_type = event.target.value;
+      this.setRecommendationType( recommendation_type )
+  }
+
+    /**
+     * Function: Set force for thamos advise
+     */
+
+     setForceParameter(thoth_force: string) {
+        if (thoth_force == "true") {
+          var force = true
+        }
+        else {
+          var force = false
+        }
+
+        this.setState(
+          {
+            thoth_force: force
+          }
+        );
+     }
+
     /**
      * Function: Set root directory where to place overlays and where to find .thoth.yaml
      */
@@ -250,23 +294,6 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
     }
 
-
-    /**
-     * Function: Set recommendation type for thamos advise
-     */
-
-    async setNewState(ui_state: IDependencyManagementUIState) {
-
-      console.log("new state", ui_state)
-      this.setState(ui_state);
-    }
-
-    changeRecommendationType(event: React .ChangeEvent<HTMLInputElement>) {
-
-        const recommendation_type = event.target.value;
-        this.setRecommendationType( recommendation_type )
-  }
-
     /**
      * Function: Set Kernel name to be created and assigned to notebook
      */
@@ -302,7 +329,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
       var thoth_timeout = Number(event.target.value)
 
       if ( Number(event.target.value) > 300 ) {
-        console.warn('kernel_name python3 cannot be used assigning default one')
+        console.warn('timeout for thoth resolution engine cannot be set more than 300 seconds due to constraint on server side.')
         var thoth_timeout: number = 300
       }
 
@@ -408,12 +435,12 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       const deleted_packages = {}
 
+      const version_deleted = _.get(packages, package_name)
       _.unset(packages, package_name)
 
       console.debug("After deleting", packages)
 
-      // TODO: Set correctly the version deleted
-      _.set(deleted_packages, package_name, "*")
+      _.set(deleted_packages, package_name, version_deleted)
 
       console.debug("Deleted", deleted_packages)
 
@@ -438,14 +465,14 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       const saved_packages = this.state.loaded_packages
 
+      const version_deleted = _.get(saved_packages, package_name)
       _.unset(saved_packages, package_name)
 
       console.debug("After deleting saved", saved_packages)
 
       const deleted_packages = {}
 
-      // TODO: Set correctly the version deleted
-      _.set(deleted_packages, package_name, "*")
+      _.set(deleted_packages, package_name, version_deleted)
 
       console.debug("Deleted", deleted_packages)
 
@@ -617,6 +644,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         var advise: Advise | undefined = await lock_requirements_with_thoth(
           this.state.kernel_name,
           this.state.thoth_timeout,
+          this.state.thoth_force,
           notebook_content,
           JSON.stringify(thoth_config),
           JSON.stringify(this.state.requirements)
@@ -635,6 +663,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         if ( advise != undefined ) {
           console.log("Thoth resolution engine error:", advise.error_msg)
         }
+        _.set(ui_state, "thoth_resolution_error_msg", advise.error_msg )
         _.set(ui_state, "resolution_engine", "pipenv" )
         _.set(ui_state, "status", "locking_requirements_using_pipenv")
         await this.setNewState(ui_state);
@@ -707,24 +736,36 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       } catch ( error ) {
 
-        console.debug("Error locking requirements with pipenv", error)
+        console.log("Error locking requirements with pipenv", error)
 
-        _.set(ui_state, "status", "failed")
-        _.set(ui_state, "error_msg", "Error asking advise to Thoth, please contact Thoth team: ")
+        if ( error.message == "Gateway error while locking dependencies with pipenv.") {
+            INotification.warning("Task was cancelled due to gateway error, try again please.")
+            _.set(ui_state, "status", "failed")
+            _.set(ui_state, "error_msg", "Gateway error, please try again!")
+            await this.setNewState(ui_state);
+            return
+        }
+
+        _.set(ui_state, "status", "failed_re")
+        _.set(ui_state, "error_msg", "Error locking requirements, check logs and please contact Thoth team: ")
         await this.setNewState(ui_state);
         return
       }
 
       if ( result == undefined || result.error == true ) {
 
+        var error_msg = "No resolution engine was able to install dependencies, please contact Thoth team: "
         if ( result != undefined ) {
-          console.log("Pipenv resolution engine error:", result.error_msg)
+          console.debug("Pipenv resolution engine error:", result.error_msg)
+          _.set(ui_state, "pipenv_resolution_error_msg", result.error_msg.toString())
+          var error_msg = `No resolution engine was able to install dependencies! Please contact Thoth team: `
         }
-        _.set(ui_state, "status", "failed")
+
+        _.set(ui_state, "status", "failed_re")
         _.set(
           ui_state,
           "error_msg",
-          "No resolution engine was able to install dependendices, please contact Thoth team: "
+          error_msg
         )
         await this.setNewState(ui_state);
         return
@@ -815,7 +856,6 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                                         <div className={CONTAINER_BUTTON}>
                                           <div className={CONTAINER_BUTTON_CENTRE}>
                                             <DependencyManagementInstallButton
-                                            changeUIstate={this.changeUIstate}
                                             install={this.lock_using_thoth} />
                                           </div>
                                         </div>
@@ -830,8 +870,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                                       <div className={CONTAINER_BUTTON}>
                                         <div className={CONTAINER_BUTTON_CENTRE}>
                                           <DependencyManagementSaveButton
-                                            onSave={this.onSave}
-                                            changeUIstate={this.changeUIstate} />
+                                            onSave={this.onSave}/>
                                         </div>
                                       </div>
                                     </div>
@@ -887,6 +926,17 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                                 onChange={this.setTimeout}
                               />
                             </label>
+                            <br />
+                            <label>
+                              Thoth --force:
+                              <select onChange={() => this.setForceParameter}>
+                                title="Thoth force parameter"
+                                name="thoth_force"
+                                value={this.state.thoth_force}
+                                <option value="false">False</option>
+                                <option value="true">True</option>
+                              </select>
+                            </label>
                           </form>
                         </div>
 
@@ -1100,11 +1150,16 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
             </div>
         );
 
-        case "failed":
+        case "failed_re":
+
+          let paragrah_failure = <p>
+            Thoth resolution engine failed because of: {this.state.thoth_resolution_error_msg} <br></br> <br></br>
+              Pipenv resolution engine failed because of: {this.state.pipenv_resolution_error_msg}<br></br> <br></br>
+              {this.state.error_msg} <br></br>
+          </p>
 
           return (
             <div>
-              {dependencyManagementform}
               <div>
                 <div className={CONTAINER_BUTTON}>
                   <div className={CONTAINER_BUTTON_CENTRE}>
@@ -1129,7 +1184,41 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                 </div>
               </div>
               <div>
-                <p>{this.state.error_msg}</p>
+                    {paragrah_failure}
+              </div>
+              <a href={"https://github.com/thoth-station/jupyterlab-requirements/issues/new?assignees=&labels=bug&template=bug_report.md"} target="_blank"> Open Issue </a>
+            </div>
+        );
+
+        case "failed":
+
+          return (
+            <div>
+              <div>
+                <div className={CONTAINER_BUTTON}>
+                  <div className={CONTAINER_BUTTON_CENTRE}>
+                    <button
+                      title='Finish.'
+                      className={OK_BUTTON_CLASS}
+                      onClick={() => this.changeUIstate(
+                        "loading",
+                        this.state.packages,
+                        this.state.loaded_packages,
+                        this.state.installed_packages,
+                        this.state.deleted_packages,
+                        this.state.requirements,
+                        this.state.kernel_name,
+                        this.state.thoth_config,
+                      )
+                      }
+                      >
+                      Ok
+                    </button>
+                  </div>
+                </div>
+              </div>
+              <div>
+                {this.state.error_msg}
               </div>
               <a href={"https://github.com/thoth-station/jupyterlab-requirements/issues/new?assignees=&labels=bug&template=bug_report.md"} target="_blank"> Open Issue </a>
             </div>
@@ -1171,6 +1260,8 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                     </p>
                 </fieldset>
               </div>
+              {addPlusInstallContainers}
+              {optionsForm}
             </div>
             );
 
@@ -1179,7 +1270,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
           // Check if kernel name is already assigned to notebook and if yes, do nothing
           const current_kernel = get_kernel_name( this.props.panel, true )
           if ( current_kernel == this.state.kernel_name ) {
-            console.log("kernel name to be assigned " + this.state.kernel_name + " already set for the notebook " + current_kernel + ", do not restart kernel...")
+            INotification.info("kernel name to be assigned " + this.state.kernel_name + " already set for the current notebook (" + current_kernel + "). Kernel won't be restarted.")
           }
           else {
             this.props.panel.sessionContext.session.changeKernel({"name": this.state.kernel_name})
