@@ -45,10 +45,20 @@ import { INotification } from "jupyterlab_toastify";
   } catch (error) {
       let message: string = error.message || error.toString();
 
-      if ( message !== 'cancelled') {
+      if ( message == 'gateway_error') {
+        console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
+        message = "Gateway error while installing dependencies.";
+      }
+      else if ( message !== 'cancelled') {
         console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
         message = `An error occurred while installing dependencies.`;
-    }
+      }
+      else {
+          console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
+          message = "Task was cancelled due to some issue.";
+      }
+
+      throw new Error(message)
   }
 }
 
@@ -126,6 +136,57 @@ export async function create_new_kernel (
   }
 }
 
+
+/**
+ * Function: Create new kernel.
+ */
+
+ export async function get_kernel_list (
+  init: RequestInit = {}
+): Promise<Array<string>> {
+
+  // GET request
+  const endpoint: string = 'kernel/create'
+
+  try {
+    const message = await requestAPI<any>(endpoint, {
+      method: 'GET'
+    });
+    return message;
+  } catch (reason) {
+    console.error('Error on GET /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', reason);
+  }
+}
+
+/**
+ * Function: Create new kernel.
+ */
+
+export async function delete_kernel (
+  kernel_name: string,
+  init: RequestInit = {}
+): Promise<Array<string>> {
+
+  // DELETE request
+  const dataToSend = {
+    kernel_name: kernel_name
+  };
+
+  const endpoint: string = 'kernel/create'
+
+  try {
+    const message = await requestAPI<any>(endpoint, {
+      body: JSON.stringify(dataToSend),
+      method: 'DELETE'
+    });
+    return message;
+  } catch (reason) {
+    console.error('Error on DELETE /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', reason);
+  }
+}
+
+
+
 /**
  * Function: Store dependencies locally.
  */
@@ -162,6 +223,34 @@ export async function store_dependencies (
   }
 
 
+/**
+ * Function: Get dependencies locally.
+ */
+
+ export async function get_dependencies (
+  kernel_name: string,
+  init: RequestInit = {},
+): Promise<string> {
+
+  // POST request
+  const dataToSend = {
+    kernel_name: kernel_name,
+  };
+
+  const endpoint: string = 'file/stored'
+
+  try {
+    const results = await requestAPI<any>(endpoint, {
+      body: JSON.stringify(dataToSend),
+      method: 'POST'
+    })
+    return JSON.parse(JSON.stringify(results));
+  } catch (reason) {
+    console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', reason);
+  }
+}
+
+
 /**
  * Function: Gather library usage from notebook cells.
  */
