@@ -32,7 +32,6 @@ export async function retrieve_config_file (
   } catch (reason) {
     console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', reason);
   }
-
 }
 
 export async function update_thoth_config_on_disk (
@@ -65,6 +64,7 @@ export async function update_thoth_config_on_disk (
 export async function lock_requirements_with_thoth(
   kernel_name: string,
   thoth_timeout: number,
+  thoth_force: boolean,
   notebook_content: string,
   thoth_config: string,
   requirements: string,
@@ -74,6 +74,7 @@ export async function lock_requirements_with_thoth(
     const dataToSend = {
       kernel_name: kernel_name,
       thoth_timeout: thoth_timeout,
+      thoth_force: thoth_force,
       notebook_content: notebook_content,
       thoth_config: thoth_config,
       requirements: requirements
@@ -97,10 +98,21 @@ export async function lock_requirements_with_thoth(
   } catch (error) {
       let message: string = error.message || error.toString();
 
-      if ( message !== 'cancelled') {
+      if ( message == 'gateway_error') {
+        console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
+        message = "Gateway error while locking dependencies with thoth.";
+      }
+      else if ( message !== 'cancelled') {
         console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
         message = `An error occurred while asking advise to Thoth.`;
-    }
+      }
+
+      else {
+        console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
+        message = "Task was cancelled due to some issue.";
+      }
+
+      throw new Error(message)
   }
 }
 
@@ -134,9 +146,19 @@ export async function lock_requirements_with_pipenv(
   } catch (error) {
       let message: string = error.message || error.toString();
 
-      if ( message !== 'cancelled') {
+      if ( message == 'gateway_error') {
         console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
-        message = `An error occurred while locking dependencies with pipenv.`;
-    }
+        message = "Gateway error while locking dependencies with pipenv.";
+      }
+      else if ( message !== 'cancelled') {
+        console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
+        message = "An error occurred while locking dependencies with pipenv.";
+      }
+      else {
+        console.error('Error on POST /' + THOTH_JUPYTER_INTEGRATION_API_BASE_NAME + '/' + endpoint + ':', message);
+        message = "Task was cancelled due to some issue.";
+      }
+
+      throw new Error(message)
   }
 }
